﻿using System;
using System.Collections.Generic;
using System.Text;
using nft.framework.plugin;
using nft.framework.drawing;
using System.Xml;
using nft.util;
using nft.framework;
using System.Drawing;
using System.IO;
using nft.core;

namespace nft.contributions.graphics
{
    public class CtbImageResource : Contribution
    {
        protected WeakReference reffer;
        protected Color colorkey;
        protected bool auto_color_key;
        protected string src_text;
        protected IFileSource file;

        public CtbImageResource(Plugin owner, ParamsReader contrib) :base(owner, contrib)
        {
            reffer = null;
            ParamsReader nd = contrib["src"];
            src_text = nd.InnerText;
            file = contrib.GetFileSource(src_text);
            ParamsReader ckey = nd["colorkey"];
            auto_color_key = ckey.IsNull;
            if (auto_color_key)
            {
                colorkey = Color.Transparent;
            }
            else
            {
                colorkey = StringParseUtil.CreateColor(ckey.InnerText);
            }
            
        }

        protected string SrcText
        {
            // 'name' holds inner text of <src> element.
            get { return src_text; }
        }

        public IFileSource File
        {
            get{ return file; }
        }

        public IImageSrc ImageSrc
        {
            get
            {
                if (reffer != null && reffer.IsAlive) {
                    return (IImageSrc)reffer.Target;
                } else {
                    // Image file might be shared by contributions in the same plugin. So assign plugin key.
                    ResourceKey key = ResourceKey.CreateKey(Parent, SrcText);
                    IImageSrc img;
                    if (auto_color_key) {
                        // use left-bottom pixel as color-key
                        img = GraphicManagerEx.GraphicManager.LoadImageFromFileWithDefaultColorKey(key, File);
                    } else {
                        img = GraphicManagerEx.GraphicManager.LoadImageFromFile(key, File, colorkey);
                    }
                    reffer = new WeakReference(img);
                    return img;
                }
            }
        }
    }
}
